Package: R2MLwiN
Version: 0.8-10
Date: 2025-10-21
Title: Running 'MLwiN' from Within R
Authors@R: c(person("Zhengzheng", "Zhang", role = c("aut", "cre"),email =
    "zhengzheng236@gmail.com"), person("Chris", "Charlton",  role = "aut"),
    person("Richard", "Parker", role = "aut"), person("George", "Leckie", role
    = "aut"), person("William", "Browne", role = "aut"))
Depends: R (>= 3.5.0),
Imports: methods, stats, stats4, Matrix, foreign (>= 0.8-46), digest
        (>= 0.6.5), texreg, foreach, parallel, doParallel, coda (>=
        0.16-1), lattice, memisc, broom, tibble
Suggests: doBy, car, lmtest, mitools, reshape, doRNG, R2WinBUGS,
        R2OpenBUGS
Description: An R command interface to the 'MLwiN' multilevel
    modelling software package.
License: GPL (>= 2)
URL: https://www.bristol.ac.uk/cmm/software/r2mlwin/,
        https://r-forge.r-project.org/projects/r2mlwin/,
        https://www.bristol.ac.uk/cmm/software/mlwin/
SystemRequirements: MLwiN, (OpenBUGS, WinBUGS optional)
Encoding: UTF-8
RoxygenNote: 7.3.3
LazyData: true
NeedsCompilation: no
Packaged: 2025-10-21 12:05:17 UTC; Administrator
Author: Zhengzheng Zhang [aut, cre],
  Chris Charlton [aut],
  Richard Parker [aut],
  George Leckie [aut],
  William Browne [aut]
Maintainer: Zhengzheng Zhang <zhengzheng236@gmail.com>
Repository: CRAN
Date/Publication: 2025-10-21 13:20:02 UTC
Built: R 4.6.0; ; 2026-04-30 15:34:55 UTC; unix
