# Generated by roxygen2: do not edit by hand

S3method(augment,mlwinfitIGLS)
S3method(augment,mlwinfitMCMC)
S3method(coef,mlwinfitIGLS)
S3method(coef,mlwinfitMCMC)
S3method(deviance,mlwinfitIGLS)
S3method(df.residual,mlwinfitIGLS)
S3method(fitted,mlwinfitIGLS)
S3method(fitted,mlwinfitMCMC)
S3method(formula,mlwinfitIGLS)
S3method(formula,mlwinfitMCMC)
S3method(getSummary,mlwinfitIGLS)
S3method(getSummary,mlwinfitMCMC)
S3method(glance,mlwinfitIGLS)
S3method(glance,mlwinfitMCMC)
S3method(logLik,mlwinfitIGLS)
S3method(nobs,mlwinfitIGLS)
S3method(nobs,mlwinfitMCMC)
S3method(predict,mlwinfitIGLS)
S3method(predict,mlwinfitMCMC)
S3method(print,mlwinfitIGLS)
S3method(print,mlwinfitMCMC)
S3method(residuals,mlwinfitIGLS)
S3method(residuals,mlwinfitMCMC)
S3method(show,mlwinfitIGLS)
S3method(show,mlwinfitMCMC)
S3method(summary,mlwinfitIGLS)
S3method(summary,mlwinfitMCMC)
S3method(tidy,mlwinfitIGLS)
S3method(tidy,mlwinfitMCMC)
S3method(update,mlwinfitIGLS)
S3method(update,mlwinfitMCMC)
S3method(vcov,mlwinfitIGLS)
S3method(vcov,mlwinfitMCMC)
export(Untoggle)
export(caterpillar)
export(caterpillarR)
export(df2matrix)
export(double2singlePrecision)
export(matrix2df)
export(mlwin2bugs)
export(predCurves)
export(predLines)
export(runMLwiN)
export(sixway)
export(trajectories)
export(ws2foreign)
exportClasses(mlwinfitIGLS)
exportClasses(mlwinfitMCMC)
exportMethods(coef)
exportMethods(extract)
exportMethods(logLik)
exportMethods(show)
exportMethods(summary)
exportMethods(update)
exportMethods(vcov)
import(doParallel)
import(foreach)
import(methods)
import(parallel)
importFrom(Matrix,nnzero)
importFrom(Matrix,sparseMatrix)
importFrom(broom,augment)
importFrom(broom,glance)
importFrom(broom,tidy)
importFrom(coda,as.mcmc.list)
importFrom(coda,effectiveSize)
importFrom(coda,is.mcmc)
importFrom(coda,mcmc)
importFrom(coda,mcmc.list)
importFrom(coda,raftery.diag)
importFrom(coda,thin)
importFrom(digest,digest)
importFrom(foreign,read.dta)
importFrom(foreign,write.dta)
importFrom(grDevices,dev.new)
importFrom(graphics,close.screen)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,screen)
importFrom(graphics,split.screen)
importFrom(graphics,text)
importFrom(lattice,Rows)
importFrom(lattice,panel.abline)
importFrom(lattice,panel.grid)
importFrom(lattice,panel.segments)
importFrom(lattice,panel.xyplot)
importFrom(lattice,qqmath)
importFrom(lattice,trellis.par.get)
importFrom(lattice,xyplot)
importFrom(memisc,getSummary)
importFrom(stats,acf)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,complete.cases)
importFrom(stats,confint)
importFrom(stats,cov)
importFrom(stats,density)
importFrom(stats,deviance)
importFrom(stats,df.residual)
importFrom(stats,end)
importFrom(stats,fitted)
importFrom(stats,fitted.values)
importFrom(stats,formula)
importFrom(stats,getCall)
importFrom(stats,get_all_vars)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,model.offset)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,pacf)
importFrom(stats,pnorm)
importFrom(stats,predict)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,reshape)
importFrom(stats,resid)
importFrom(stats,residuals)
importFrom(stats,sd)
importFrom(stats,start)
importFrom(stats,symnum)
importFrom(stats,terms)
importFrom(stats,terms.formula)
importFrom(stats,update)
importFrom(stats,update.formula)
importFrom(stats,var)
importFrom(stats,vcov)
importFrom(stats,window)
importFrom(tibble,tibble)
importFrom(utils,citation)
importFrom(utils,read.delim)
importFrom(utils,stack)
importMethodsFrom(stats4,coef)
importMethodsFrom(stats4,logLik)
importMethodsFrom(stats4,show)
importMethodsFrom(stats4,summary)
importMethodsFrom(stats4,update)
importMethodsFrom(stats4,vcov)
importMethodsFrom(texreg,extract)
